package frc.robot.subsystems;

import com.revrobotics.RelativeEncoder;
import com.revrobotics.spark.ClosedLoopSlot;
import com.revrobotics.spark.SparkClosedLoopController;
import com.revrobotics.spark.SparkMax;
import com.revrobotics.spark.SparkBase.ControlType;
import com.revrobotics.spark.SparkBase.PersistMode;
import com.revrobotics.spark.SparkBase.ResetMode;
import com.revrobotics.spark.SparkClosedLoopController.ArbFFUnits;
import com.revrobotics.spark.SparkLowLevel.MotorType;

import edu.wpi.first.math.controller.ElevatorFeedforward;
import edu.wpi.first.math.trajectory.TrapezoidProfile;
import edu.wpi.first.math.trajectory.TrapezoidProfile.Constraints;
import edu.wpi.first.wpilibj2.command.Command;
import edu.wpi.first.wpilibj2.command.SubsystemBase;
import static frc.robot.constants.SemiClosedLoopConstants.*;

public class SemiClosedLoop extends SubsystemBase {

    private RelativeEncoder encoder;

    private SparkClosedLoopController controller;

    private SparkMax motor1;

    private TrapezoidProfile trapezoidProfile;

    private TrapezoidProfile.State currentGoal;
    private TrapezoidProfile.State setpoint;

    private ElevatorFeedforward elevatorFeedforward;

    public SemiClosedLoop() {
        motor1 = new SparkMax(kMotorID, MotorType.kBrushless);

        motor1.configure(
            motorConfig, 
            ResetMode.kResetSafeParameters, 
            PersistMode.kPersistParameters
        );

        encoder = motor1.getEncoder();

        encoder.setPosition(0);

        controller = motor1.getClosedLoopController();

        trapezoidProfile = new TrapezoidProfile(
            new Constraints(
                kMaxVelocity, 
                kMaxAcceleration
            )
        );
    }

    public Command goToPosition(double goal) {
        return startRun(
            () -> {
                setpoint = new TrapezoidProfile.State(encoder.getPosition(), encoder.getVelocity());
                currentGoal = new TrapezoidProfile.State(goal, 0);
            }, 
            () -> {
                setpoint = trapezoidProfile.calculate(0.02, setpoint, currentGoal);

                controller.setReference(
                    setpoint.position, 
                    ControlType.kPosition, 
                    ClosedLoopSlot.kSlot0
                );
            }
        );
    }

    public Command goToVelocity(double velocity) {
        return startRun(
        () -> {

        }, 
        () -> {

        });
    }

    public double getEncoderPosition() {
        return encoder.getPosition();
    }
}
